package org.xbib.time.chronic;

import org.xbib.time.chronic.tags.Tag;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 *
 */
public class Token {

    private final String word;

    private final List<Tag<?>> tags;

    public Token(String word) {
        this.word = word;
        tags = new LinkedList<>();
    }

    public String getWord() {
        return word;
    }

    /**
     * Tag this token with the specified tag.
     * @param newTag new tag
     */
    public void tag(Tag<?> newTag) {
        tags.add(newTag);
    }

    /**
     * Remove all tags of the given class.
     * @param tagClass tag class
     */
    public void untag(Class<?> tagClass) {
        tags.removeIf(tagClass::isInstance);
    }

    /**
     * Return true if this token has any tags.
     * @return true if token has tags
     */
    public boolean isTagged() {
        return !tags.isEmpty();
    }

    /**
     * Return the Tag that matches the given class.
     * @param tagClass tag class
     * @param <T> type parameter
     * @return tag
     */
    public <T extends Tag<?>> T getTag(Class<T> tagClass) {
        List<T> matches = getTags(tagClass);
        T matchingTag = null;
        if (!matches.isEmpty()) {
            matchingTag = matches.get(0);
        }
        return matchingTag;
    }

    public List<Tag<?>> getTags() {
        return tags;
    }

    /**
     *  Return the Tag that matches the given class.
     *
     * @param tagClass tag class
     * @param <T> type parameter
     * @return list of tags
     */
    @SuppressWarnings("unchecked")
    public <T extends Tag<?>> List<T> getTags(Class<T> tagClass) {
        List<T> matches = new LinkedList<>();
        for (Tag<?> tag : tags) {
            if (tagClass.isInstance(tag)) {
                matches.add((T) tag);
            }
        }
        return matches;
    }

    @Override
    public String toString() {
        return word + " " + tags;
    }
}
