package org.xbib.time.chronic.repeaters;

import org.xbib.time.chronic.Span;
import org.xbib.time.chronic.tags.PointerType;

import java.time.ZonedDateTime;
import java.time.temporal.ChronoUnit;

/**
 *
 */
public class RepeaterWeek extends RepeaterUnit {

    public static final int WEEK_SECONDS = 604800;

    public static final int WEEK_DAYS = 7;

    private ZonedDateTime currentWeekStart;

    private static ZonedDateTime ymdh(ZonedDateTime zonedDateTime) {
        return ZonedDateTime.of(zonedDateTime.getYear(), zonedDateTime.getMonthValue(), zonedDateTime.getDayOfMonth(),
                zonedDateTime.getHour(), 0, 0, 0, zonedDateTime.getZone());
    }

    @Override
    protected Span internalNextSpan(PointerType pointer) {
        if (currentWeekStart == null) {
            if (pointer == PointerType.FUTURE) {
                RepeaterDayName sundayRepeater = new RepeaterDayName(DayName.SUNDAY);
                sundayRepeater.setNow(getNow());
                Span nextSundaySpan = sundayRepeater.nextSpan(PointerType.FUTURE);
                currentWeekStart = nextSundaySpan.getBeginCalendar();
            } else if (pointer == PointerType.PAST) {
                RepeaterDayName sundayRepeater = new RepeaterDayName(DayName.SUNDAY);
                sundayRepeater.setNow(getNow().plus(1, ChronoUnit.DAYS));
                sundayRepeater.nextSpan(PointerType.PAST);
                Span lastSundaySpan = sundayRepeater.nextSpan(PointerType.PAST);
                currentWeekStart = lastSundaySpan.getBeginCalendar();
            } else {
                throw new IllegalArgumentException("Unable to handle pointer " + pointer + ".");
            }
        } else {
            long direction = pointer == PointerType.FUTURE ? 1L : -1L;
            currentWeekStart = currentWeekStart.plus(RepeaterWeek.WEEK_DAYS * direction, ChronoUnit.DAYS);
        }

        return new Span(currentWeekStart, ChronoUnit.DAYS, RepeaterWeek.WEEK_DAYS);
    }

    @Override
    protected Span internalThisSpan(PointerType pointer) {
        Span thisWeekSpan;
        ZonedDateTime thisWeekStart;
        ZonedDateTime thisWeekEnd;
        if (pointer == PointerType.FUTURE) {
            thisWeekStart = ymdh(getNow()).plus(1, ChronoUnit.HOURS);
            RepeaterDayName sundayRepeater = new RepeaterDayName(DayName.SUNDAY);
            sundayRepeater.setNow(getNow());
            Span thisSundaySpan = sundayRepeater.thisSpan(PointerType.FUTURE);
            thisWeekEnd = thisSundaySpan.getBeginCalendar();
            thisWeekSpan = new Span(thisWeekStart, thisWeekEnd);
        } else if (pointer == PointerType.PAST) {
            thisWeekEnd = ymdh(getNow());
            RepeaterDayName sundayRepeater = new RepeaterDayName(DayName.SUNDAY);
            sundayRepeater.setNow(getNow());
            Span lastSundaySpan = sundayRepeater.nextSpan(PointerType.PAST);
            thisWeekStart = lastSundaySpan.getBeginCalendar();
            thisWeekSpan = new Span(thisWeekStart, thisWeekEnd);
        } else if (pointer == PointerType.NONE) {
            RepeaterDayName sundayRepeater = new RepeaterDayName(DayName.SUNDAY);
            sundayRepeater.setNow(getNow());
            Span lastSundaySpan = sundayRepeater.nextSpan(PointerType.PAST);
            thisWeekStart = lastSundaySpan.getBeginCalendar();
            thisWeekEnd = thisWeekStart.plus(RepeaterWeek.WEEK_DAYS, ChronoUnit.DAYS);
            thisWeekSpan = new Span(thisWeekStart, thisWeekEnd);
        } else {
            throw new IllegalArgumentException("Unable to handle pointer " + pointer + ".");
        }
        return thisWeekSpan;
    }

    @Override
    public Span getOffset(Span span, int amount, PointerType pointer) {
        long direction = pointer == PointerType.FUTURE ? 1L : -1L;
        return span.add(direction * amount * RepeaterWeek.WEEK_SECONDS);
    }

    @Override
    public int getWidth() {
        return RepeaterWeek.WEEK_SECONDS;
    }

    @Override
    public int hashCode() {
        return super.hashCode() ^ getWidth();
    }

    @Override
    public boolean equals(Object other) {
        return other instanceof RepeaterWeek &&
                ((Repeater) other).getType().equals(getType()) &&
                ((Repeater) other).getNow().equals(getNow());
    }

    @Override
    public String toString() {
        return super.toString() + "-week";
    }

}
